version 1.0

import "wf_mapping.wdl"
import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/basemods.wdl" as basemods

workflow pb_basemods {
  input {
    File eid_subread
    File eid_ref_dataset
    Boolean run_find_motifs = false

    String dataset_filters = ""
    Int downsample_factor = 0
    String kineticstools_identify_mods = "m4C,m6A"
    Float kineticstools_p_value = 0.001
    Boolean kineticstools_compute_methyl_fraction = false
    Float motif_min_score = 30.0
    Float motif_min_fraction = 0.30
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    String? mapping_biosample_name
    String? mapping_pbmm2_overrides
    Boolean consolidate_aligned_bam = false

    Int nproc = 1
    Int max_nchunks = 100
    Int target_size = 20000
    String log_level = "INFO"
    String? tmp_dir
  }

  Boolean write_bigwig = true
  Int max_genome_size_for_csv = 10000000

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_reference {
    input:
      dataset = eid_ref_dataset,
      log_level = log_level,
      nproc = 1
  }

  call basemods.get_dataset_size {
    input:
      dataset = update_reference.updated_xml,
      nproc = 1,
      log_level = log_level
  }

  Boolean write_csv = get_dataset_size.dataset_size <= max_genome_size_for_csv

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1
  }

  call wf_mapping.mapping {
    input:
      reads = dataset_filter.filtered,
      reference = update_reference.updated_xml,
      biosample_name = mapping_biosample_name,
      pbmm2_overrides = mapping_pbmm2_overrides,
      min_concordance = mapping_min_concordance,
      min_length = mapping_min_length,
      run_coverage = false,
      run_bam_auto_consolidation = false,
      nproc = nproc,
      log_level = log_level,
      max_nchunks = max_nchunks,
      target_size = target_size,
      tmp_dir = tmp_dir
  }

  # the coverage tasks are run in the main workflow because they are blocking
  # otherwise (due to the way Cromwell executes sub-workflows)
  call pbreports.summarize_coverage as summarize_coverage {
    input:
      mapped = mapping.mapped,
      reference = update_reference.updated_xml,
      nproc = nproc
  }

  call pbreports.coverage as coverage_report {
    input:
      reference = update_reference.updated_xml,
      coverage_gff = summarize_coverage.coverage_gff,
      nproc = nproc
  }

  # this may take a while but it runs concurrently with ipdSummary
  call pbcoretools.auto_consolidate_alignments {
    input:
      mapped = mapping.mapped,
      force_consolidate = consolidate_aligned_bam,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  call chunking.split_alignments {
    input:
      mapped = mapping.mapped,
      max_nchunks = max_nchunks
  }

  scatter (alignments in split_alignments.chunks) {
    call basemods.ipdsummary {
      input:
        mapped = alignments,
        reference = update_reference.updated_xml,
        identify_mods = kineticstools_identify_mods,
        p_value = kineticstools_p_value,
        compute_methyl_fraction = kineticstools_compute_methyl_fraction,
        write_bigwig = write_bigwig,
        write_csv = write_csv,
        nproc = nproc,
        log_level = log_level
    }
  }

  call chunking.gather_generic as gather_basemods_gff {
    input:
      chunks = ipdsummary.basemods_gff,
      output_file_name = "basemods.gff",
      log_level = log_level
  }

  if (write_bigwig) {
    call basemods.gather_bigwig {
      input:
        bigwig_files = select_all(ipdsummary.bigwig_file),
        log_level = log_level
    }
  }

  if (write_csv) {
    call chunking.gather_generic as gather_basemods_csv {
      input:
        chunks = select_all(ipdsummary.basemods_csv),
        output_file_name = "basemods.csv",
        log_level = log_level
    }

    call pbreports.modifications as basemods_report {
      input:
        basemods_csv = gather_basemods_csv.gathered,
        nproc = nproc,
        log_level = log_level
    }
  }

  if (run_find_motifs) {
    call basemods.find_motifs {
      input:
        basemods_gff = gather_basemods_gff.gathered,
        reference = update_reference.updated_xml,
        min_score = motif_min_score,
        nproc = nproc,
        log_level = log_level
    }

    call basemods.reprocess_motifs {
      input:
        basemods_gff = gather_basemods_gff.gathered,
        motifs_csv = find_motifs.motifs_csv,
        reference = update_reference.updated_xml,
        min_fraction = motif_min_fraction
    }

    call pbreports.motifs as motifs_report {
      input:
        motifs_gff = reprocess_motifs.motifs_gff,
        motifs_csv = find_motifs.motifs_csv
    }
  }

  output {
    File mapped = mapping.mapped
    File? report_mapping_stats = mapping.report_mapping_stats
    File coverage_gff = summarize_coverage.coverage_gff
    File report_coverage = coverage_report.report
    File basemods_gff = gather_basemods_gff.gathered
    File? basemods_csv = gather_basemods_csv.gathered
    File? report_modifications = basemods_report.report
    File? bigwig_file = gather_bigwig.gathered
    File? motifs_csv = find_motifs.motifs_csv
    File? motifs_gff = reprocess_motifs.motifs_gff
    File? report_motifs = motifs_report.report
    File? mapped_bam_datastore = auto_consolidate_alignments.datastore
  }
}
