# HGAP4
version 1.0

import "wf_falcon.wdl"
import "wf_consensus.wdl"
import "wf_mapping.wdl"
import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/pbreports.wdl" as pbreports

task task_get_fastas {
  input {
    File fofn # for now, all abspaths
  }
  Array[File] fns = read_lines(fofn)

  command <<<
  >>>
  output {
    Array[File] array_of_fasta = fns
  }
}

task task_get_dextas {
  input {
    File subreadset_xml # can have multiple BAMs
    Int nproc = 1
  }
  command <<<
    set -vex
    env | sort

    # dataset absolutize --outdir . ... # would fail because it will not traverse the symlink
    rfn=$(readlink -f ~{subreadset_xml})
    python3 -m falcon_kit.mains.bam2dexta --nproc=~{nproc}  split-apply-combine \
        --bam=${rfn} --dexta=./dexta.fofn

    cat ./dexta.fofn
    ls -l uow-*/*.dexta
  >>>
  runtime {
    cpu: nproc
  }
  output {
    Array[File] array_of_dexta = glob("uow-*/*.dexta")
  }
}
task task_gen_config {
  input {
    Array[File] array_of_fasta # can be .dexta or .fasta files
    # These are effectively workflow-options.
    Boolean HGAP_AggressiveAsm_bool
    String HGAP_GenomeLength_str
    String HGAP_SeedCoverage_str
    String HGAP_SeedLengthCutoff_str
    String HGAP_FalconAdvanced_str
  }
  Object object_opts = object {
    HGAP_AggressiveAsm_bool: HGAP_AggressiveAsm_bool,
    HGAP_GenomeLength_str: HGAP_GenomeLength_str,
    HGAP_SeedCoverage_str: HGAP_SeedCoverage_str,
    HGAP_SeedLengthCutoff_str: HGAP_SeedLengthCutoff_str,
    HGAP_FalconAdvanced_str: HGAP_FalconAdvanced_str,
  }
  File hgap4_options_json = write_json(object_opts)
  command <<<
    set -vex
    ln -f ~{hgap4_options_json} ./hgap4_options.json
    python3 -m pbfalcon.gen_config --i-options-fn ./hgap4_options.json --o-cfg-fn ./fc_run.cfg --o-json-fn ./General_config.json
  >>>
  output {
    File cfg = "fc_run.cfg"
    File config = "General_config.json"
  }
}

task fasta_to_reference {
  input {
    File fasta

    String log_level = "INFO"
  }
  command {
    set -vex
    ln -s ${fasta} falcon_draft_assembly.fasta
    dataset \
      --log-level ${log_level} \
      create \
      --generateIndices \
      --type ReferenceSet \
      --name falcon_draft_assembly \
      falcon_draft_assembly.referenceset.xml \
      falcon_draft_assembly.fasta
  }
  runtime {
    cpu: 1
  }
  output {
    File reference_xml = "falcon_draft_assembly.referenceset.xml"
  }
}

workflow pb_hgap4 {
  input {
    File eid_subread
    #File ifile_input_fofn = "/scratch/cdunn/pbcromwell/abs_input.fofn"
    ##File ifile_config = "/scratch/cdunn/pbcromwell/General_config.json"
    ##File ifile_config = "General_config.json"
    Boolean HGAP_AggressiveAsm_bool  = false
    String HGAP_GenomeLength_str     = "5000000"
    String HGAP_SeedCoverage_str     = "30"
    String HGAP_SeedLengthCutoff_str = "-1"
    String HGAP_FalconAdvanced_str   = ""
    String consensus_algorithm = "best"
    String dataset_filters = ""
    Int downsample_factor = 0
    String? mapping_biosample_name
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    String? mapping_pbmm2_overrides
    Boolean consolidate_aligned_bam = false

    Int nproc = 8
    Int max_nchunks = 100
    String? tmp_dir
    String log_level = "INFO"
  }

  ##Map[String, String] cfg = read_json(ifile_config)
  ##Array[File] array_of_fasta = read_lines(ifile_input_fofn)

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }
  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      #nproc = 1 # currently ignored anyway
  }
  #call task_get_fastas {
  #  input:
  #    fofn = ifile_input_fofn,
  #}
  call task_get_dextas {
    input:
      #subreadset_xml = pbcoretools.dataset_filter.filtered, # does not work
      subreadset_xml = dataset_filter.filtered,
      nproc          = nproc,
  }
  call task_gen_config {
    input:
      #array_of_fasta = task_get_fastas.array_of_fasta,
      array_of_fasta = task_get_dextas.array_of_dexta,  # can be dexta or fasta
      HGAP_AggressiveAsm_bool = HGAP_AggressiveAsm_bool,
      HGAP_GenomeLength_str = HGAP_GenomeLength_str,
      HGAP_SeedCoverage_str = HGAP_SeedCoverage_str,
      HGAP_SeedLengthCutoff_str = HGAP_SeedLengthCutoff_str,
      HGAP_FalconAdvanced_str = HGAP_FalconAdvanced_str
  }
  call wf_falcon.falcon {
    input:
      ifile_config   = task_gen_config.config,
      #array_of_fasta = task_get_fastas.array_of_fasta,
      array_of_fasta = task_get_dextas.array_of_dexta,
      nproc          = nproc,
      max_nchunks    = max_nchunks
  }

  call fasta_to_reference {
    input:
      fasta = falcon.ofile_p_ctg_fasta,
      log_level = log_level
  }

  call wf_mapping.mapping {
    input:
      reads = dataset_filter.filtered,
      reference = fasta_to_reference.reference_xml,
      mapping_stats_module = "mapping_stats_hgap",
      biosample_name = mapping_biosample_name,
      min_concordance = mapping_min_concordance,
      min_length = mapping_min_length,
      pbmm2_overrides = mapping_pbmm2_overrides,
      run_coverage = false,
      run_bam_auto_consolidation = false,
      nproc = nproc,
      max_nchunks =  max_nchunks,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  call pbcoretools.auto_consolidate_alignments {
    input:
      mapped = mapping.mapped,
      force_consolidate = consolidate_aligned_bam,
      tmp_dir = tmp_dir,
      log_level = log_level
  }

  # the coverage tasks are run in the main workflow because they are blocking
  # otherwise (due to the way Cromwell executes sub-workflows)
  call pbreports.summarize_coverage as summarize_coverage {
    input:
      mapped = mapping.mapped,
      reference = fasta_to_reference.reference_xml,
      nproc = nproc
  }

  call pbreports.coverage as coverage_report {
    input:
      reference = fasta_to_reference.reference_xml,
      coverage_gff = summarize_coverage.coverage_gff,
      report_module = "coverage_hgap",
      nproc = nproc
  }

  call wf_consensus.consensus {
    input:
      alignments = mapping.mapped,
      reference = fasta_to_reference.reference_xml,
      consensus_algorithm = "arrow",
      log_level = log_level,
      nproc = nproc,
      max_nchunks = max_nchunks
  }

  call pbreports.polished_assembly {
    input:
      coverage_gff = summarize_coverage.coverage_gff,
      consensus_fastq = consensus.consensus_fastq,
      nproc = nproc,
      log_level = log_level
  }

  output {
    File ofile_p_ctg_fasta = falcon.ofile_p_ctg_fasta
    File ofile_a_ctg_fasta = falcon.ofile_a_ctg_fasta
    File mapped = mapping.mapped
    File coverage_gff = summarize_coverage.coverage_gff
    File? report_mapping_stats = mapping.report_mapping_stats
    File report_coverage = coverage_report.report
    File consensus_fasta = consensus.consensus_fasta
    File consensus_fastq = consensus.consensus_fastq
    File report_preassembly = falcon.report_preassembly
    File report_polished_assembly = polished_assembly.report
    File? mapped_bam_datastore = auto_consolidate_alignments.datastore
  }
}
