# Parallelized Iso-Seq 3 workflow with optional minimap2 alignment

version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbmm2.wdl" as mapping
import "tasks/pbreports.wdl" as pbreports
import "tasks/barcoding.wdl" as barcoding
import "tasks/isoseq3.wdl" as isoseq3
import "tasks/unanimity.wdl" as unanimity
import "tasks/bam2fastx.wdl" as bam2fastx 

workflow pb_isoseq3 {
  input {
    File eid_subread
    File eid_barcode
    File? eid_ref_dataset

    # Optional tasks
    Boolean run_clustering = true

    # Options exposed in smrtlink: 
    String dataset_filters = ""
    Boolean ccs_polish = false
    Int ccs_min_length = 50
    Int ccs_max_length = 15000

    Boolean lima_ignore_biosamples = false

    Boolean refine_require_polya = true
    Float hq_cutoff = 0.99

    Int mapping_min_length = 50
    Float mapping_min_concordance = 95
    Float mapping_min_coverage = 99

    Int isocollapse_max_fuzzy_junction = 5

    Int nproc = 1
    String log_level = "INFO"
    Int max_nchunks = 100
    String? tmp_dir
  }

  # Options hidden (not exposed) in smrtlink
  Int downsample_factor = 0
  Int ccs_min_passes = 1
  Float ccs_min_predicted_accuracy = 0.80
  Int ccs_max_poa_coverage = 10
  String ccs_draft_mode_args = "--draft-mode winpoa"
  Boolean ccs_disable_heuristics = true

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_barcodes {
    input:
      dataset = eid_barcode,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1
  }

  # CCS (with chunking)
  call chunking.split_dataset as split_subreads {
    input:
      ds_in = dataset_filter.filtered,
      max_nchunks = max_nchunks
  }

  scatter (subreads in split_subreads.chunks) {
    call unanimity.ccs as ccs_isoseq {
      input:
        subreads = subreads,
        polishing = ccs_polish,
        min_passes = ccs_min_passes,
        min_predicted_accuracy = ccs_min_predicted_accuracy,
        max_poa_coverage = ccs_max_poa_coverage,
        draft_mode_args = ccs_draft_mode_args,
        disable_heuristics = ccs_disable_heuristics,
        min_length =  ccs_min_length,
        max_length = ccs_max_length,
        nproc = nproc
    }
  }

  call chunking.gather_datasets as gather_ccsxml {
    input:
      chunks = ccs_isoseq.ccsxml,
      dataset_name = "'CCS Data'",
      dataset_ext = ".consensusreadset.xml"
  }

  call pbreports.ccs1 as pbreports_ccs_report {
    input:
      ccsxml = gather_ccsxml.gathered,
      nproc = nproc
  }

  call bam2fastx.bam2fastq_archive as bam2fastq_ccs {
    input:
      reads = gather_ccsxml.gathered,
      prefix = "ccs"
  }

  # "Barcoding", also read classification
  # TODO is this chunkable since we don't use peek/guess?
  call barcoding.lima_isoseq {
    input:
      ccsxml = gather_ccsxml.gathered,
      barcodes = update_barcodes.updated_xml,
      ignore_biosamples = lima_ignore_biosamples,
      nproc = nproc
  }

  call barcoding.datastore_to_merged_dataset {
    input:
      datastore = lima_isoseq.datastore,
      dataset_ext = ".consensusreadset.xml",
      nproc = nproc,
      log_level = log_level
  }

  call isoseq3.refine as isoseq3_refine {
    input:
      fl_ccs = datastore_to_merged_dataset.mergedxml,
      barcodes = update_barcodes.updated_xml,
      require_polya = refine_require_polya,
      nproc = nproc
  }

  call pbreports.barcode_isoseq3 as pbreports_barcode_isoseq3 {
    input:
      barcoded = lima_isoseq.datastore,
      unbarcoded = gather_ccsxml.gathered,
      barcodes = update_barcodes.updated_xml,
      filter_summary = isoseq3_refine.filter_summary,
      flnc_ccs = isoseq3_refine.flnc_ccs
  }

  # This was traditionally optional, because Blasr.  I think minimap2 makes
  # the classify-only pipeline obsolete.
  if (run_clustering) {

    # Transcript clustering (all-vs-all, not chunkable)
    call isoseq3.cluster as isoseq3_cluster {
      input:
        flnc_ccs = isoseq3_refine.flnc_ccs,
        nproc = nproc
    }

    # Transcript polishing with input subreads
    # (note that isoseq3_cluster generates sequential pseudo-ZMW-hole-numbers)
    call chunking.split_dataset as split_transcripts {
      input:
        ds_in = isoseq3_cluster.unpolished,
        target_size = 1000,
        max_nchunks = max_nchunks
    }

    scatter (unpolished in split_transcripts.chunks) {
      call isoseq3.polish as isoseq3_polish {
        input:
          unpolished = unpolished,
          subreads = dataset_filter.filtered,
          nproc = nproc
      }
    }

    call chunking.gather_datasets as gather_transcripts {
      input:
        chunks = isoseq3_polish.polished,
        dataset_name = "'Polished Transcripts'",
        dataset_ext = ".transcriptset.xml"
    }

    call chunking.gather_generic as gather_polish_report {
      input:
        chunks = isoseq3_polish.report_csv,
        output_file_name = "polished.cluster_report.csv"
    }

    call isoseq3.summarize as isoseq3_summarize {
      input:
        transcriptset = gather_transcripts.gathered,
        barcodeset = update_barcodes.updated_xml
    }

    call isoseq3.extract_hq_transcripts {
      input:
        polished = gather_transcripts.gathered,
        hq_cutoff = hq_cutoff
    }

    call pbreports.isoseq3 as isoseq_report {
      input:
        hq_transcripts = extract_hq_transcripts.hq_transcripts,
        lq_transcripts = extract_hq_transcripts.lq_transcripts
    }

    call isoseq3.bam2fastq_transcripts {
      input:
        hq_transcripts = extract_hq_transcripts.hq_transcripts,
        lq_transcripts = extract_hq_transcripts.lq_transcripts,
        subreads = dataset_filter.filtered
    }

    call isoseq3.bam2fasta_transcripts {
      input:
        hq_transcripts = extract_hq_transcripts.hq_transcripts,
        lq_transcripts = extract_hq_transcripts.lq_transcripts,
        subreads = dataset_filter.filtered
    }

    call pbcoretools.consolidate_transcripts {
      input:
        subreads = update_subreads.updated_xml,
        hq_transcripts = extract_hq_transcripts.hq_transcripts,
        lq_transcripts = extract_hq_transcripts.lq_transcripts
    }

    # Optional mapping if reference is defined
    if (defined(eid_ref_dataset)) {

      call pbcoretools.dataset_absolutize as update_reference {
        input:
          dataset = eid_ref_dataset,
          log_level = log_level,
          nproc = 1
      }

      call chunking.split_dataset as split_transcripts2 {
        input:
          ds_in = consolidate_transcripts.out_hq_transcripts,
          max_nchunks = max_nchunks,
          target_size = 1000
      }

      scatter (transcripts in split_transcripts2.chunks) {
        call mapping.pbmm2_align as pbmm2_align {
          input:
            unmapped = transcripts,
            reference = update_reference.updated_xml,
            aln_ext = ".transcriptalignmentset.xml",
            min_concordance = mapping_min_concordance,
            min_length = mapping_min_length,
            preset_mode = "ISOSEQ",
            nproc = nproc
        }
      }

      call chunking.gather_datasets as gather_transcriptalignments {
        input:
          chunks = pbmm2_align.mapped,
          dataset_name = "'Mapped Transcripts'",
          dataset_ext = ".transcriptalignmentset.xml"
      }

      call pbcoretools.consolidate_alignments as consolidate_transcriptaln {
        input:
          mapped = gather_transcriptalignments.gathered,
          log_level = log_level
      }

      call isoseq3.collapse as isoseq3_collapse {
        input:
          hq_transcriptaln_bam = consolidate_transcriptaln.mapped_bam,
          flnc_bam = isoseq3_refine.flnc_bam,
          flnc_bam_pbi = isoseq3_refine.flnc_bam_pbi,
          max_fuzzy_junction = isocollapse_max_fuzzy_junction,
          min_aln_identity = mapping_min_concordance / 100.0,
          min_aln_coverage = mapping_min_coverage / 100.0,
          nproc = nproc,
          log_level = log_level
      }

      call pbreports.isoseq_mapping as isoseq_mapping_report {
        input:
          collapse_fasta_or_fastq = isoseq3_collapse.fastq,
          collapse_report_json = isoseq3_collapse.report
      }
    }
  }

  output {
    File ccs_fastq_zip = bam2fastq_ccs.fastq_zip
    File flnc_bam = isoseq3_refine.flnc_bam
    File flnc_report = isoseq3_refine.report_csv
    File report_ccs = pbreports_ccs_report.report
    File report_barcode_isoseq3 = pbreports_barcode_isoseq3.report
    File? report_isoseq = isoseq_report.report
    File? polish_report_csv = gather_polish_report.gathered
    File? barcode_overview_report = isoseq3_summarize.report_csv
    File? hq_fastq = bam2fastq_transcripts.hq_fastq
    File? lq_fastq = bam2fastq_transcripts.lq_fastq
    File? hq_fasta = bam2fasta_transcripts.hq_fasta
    File? lq_fasta = bam2fasta_transcripts.lq_fasta
    File? hq_aln_bam = consolidate_transcriptaln.mapped_bam
    File? hq_aln_bai = consolidate_transcriptaln.mapped_bam_bai
    File? collapse_fastq = isoseq3_collapse.fastq
    File? collapse_gff = isoseq3_collapse.gff
    File? collapse_group = isoseq3_collapse.group
    File? collapse_abundance = isoseq3_collapse.abundance
    File? collapse_readstat = isoseq3_collapse.readstat
    File? report_isoseq_mapping = isoseq_mapping_report.report
  }
}
