# Parallelized Iso-Seq 3 workflow with optional minimap2 alignment

version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbmm2.wdl" as mapping
import "tasks/pbreports.wdl" as pbreports
import "tasks/barcoding.wdl" as barcoding
import "tasks/isoseq3.wdl" as isoseq3
import "tasks/bam2fastx.wdl" as bam2fastx 


workflow pb_isoseq3_ccsonly {
  input {
    File eid_ccs 
    File eid_barcode
    File? eid_ref_dataset

    # Optional tasks
    Boolean run_clustering = true

    # Options exposed in smrtlink: 
    Boolean refine_require_polya = true
    Int min_subreads_split = 7  # Minimum subreads number to flag transcript as HQ

    Int mapping_min_length = 50
    Float mapping_min_concordance = 95
    Float mapping_min_coverage = 99

    Int isocollapse_max_fuzzy_junction = 5

    Int nproc = 1
    String log_level = "INFO"
    Int max_nchunks = 100
    String? tmp_dir
  }

  call pbcoretools.dataset_absolutize as update_reads {
    input:
      dataset = eid_ccs,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_barcodes {
    input:
      dataset = eid_barcode,
      log_level = log_level,
      nproc = 1
  }

  # Check input ccs bam is not demultiplexed
  call isoseq3.check_reads_for_lima as check_ccs {
    input: 
      ccs = eid_ccs,
      log_level = log_level,
      nproc = 1
  }

  # "Barcoding", also read classification
  call barcoding.lima_isoseq {
    input:
      ccsxml = eid_ccs,
      barcodes = update_barcodes.updated_xml,
      nproc = nproc
  }

  call barcoding.datastore_to_merged_dataset {
    input:
      datastore = lima_isoseq.datastore,
      dataset_ext = ".consensusreadset.xml",
      nproc = nproc,
      log_level = log_level
  }

  call isoseq3.refine as isoseq3_refine {
    input:
      fl_ccs = datastore_to_merged_dataset.mergedxml,
      barcodes = update_barcodes.updated_xml,
      require_polya = refine_require_polya,
      nproc = nproc
  }

  call pbreports.barcode_isoseq3 as pbreports_barcode_isoseq3 {
    input:
      barcoded = lima_isoseq.datastore,
      unbarcoded = eid_ccs,
      barcodes = update_barcodes.updated_xml,
      filter_summary = isoseq3_refine.filter_summary,
      flnc_ccs = isoseq3_refine.flnc_ccs
  }

  if (run_clustering) {
    call isoseq3.cluster_ccsonly as isoseq3_cluster {
      input:
        flnc_ccs = isoseq3_refine.flnc_ccs,
        min_subreads_split = min_subreads_split,
        nproc = nproc
    }

    call isoseq3.summarize as isoseq3_summarize {
      input:
        transcriptset = isoseq3_cluster.unpolished,
        barcodeset = update_barcodes.updated_xml
    }

    call pbreports.isoseq3 as isoseq_report {
      input:
        hq_transcripts = isoseq3_cluster.hq_transcripts,
        lq_transcripts = isoseq3_cluster.lq_transcripts
    }

    call isoseq3.bam2fasta_transcripts {
      input:
        hq_transcripts = isoseq3_cluster.hq_transcripts,
        lq_transcripts = isoseq3_cluster.lq_transcripts,
        subreads = eid_ccs
    }

    call pbcoretools.consolidate_transcripts {
      input:
        subreads = update_reads.updated_xml,
        hq_transcripts = isoseq3_cluster.hq_transcripts,
        lq_transcripts = isoseq3_cluster.lq_transcripts
    }

    # Optional mapping if reference is defined
    if (defined(eid_ref_dataset)) {

      call pbcoretools.dataset_absolutize as update_reference {
        input:
          dataset = eid_ref_dataset,
          log_level = log_level,
          nproc = 1
      }

      call chunking.split_dataset as split_transcripts2 {
        input:
          ds_in = consolidate_transcripts.out_hq_transcripts,
          max_nchunks = max_nchunks,
          target_size = 1000
      }

      scatter (transcripts in split_transcripts2.chunks) {
        call mapping.pbmm2_align as pbmm2_align {
          input:
            unmapped = transcripts,
            reference = update_reference.updated_xml,
            aln_ext = ".transcriptalignmentset.xml",
            min_concordance = mapping_min_concordance,
            min_length = mapping_min_length,
            preset_mode = "ISOSEQ",
            nproc = nproc
        }
      }

      call chunking.gather_datasets as gather_transcriptalignments {
        input:
          chunks = pbmm2_align.mapped,
          dataset_name = "'Mapped Transcripts'",
          dataset_ext = ".transcriptalignmentset.xml"
      }

      call pbcoretools.consolidate_alignments as consolidate_transcriptaln {
        input:
          mapped = gather_transcriptalignments.gathered,
          log_level = log_level,
          tmp_dir = tmp_dir
      }

      call isoseq3.collapse_ccsonly as isoseq3_collapse {
        input:
          hq_transcriptaln_bam = consolidate_transcriptaln.mapped_bam,
          flnc_bam = isoseq3_refine.flnc_bam,
          flnc_bam_pbi = isoseq3_refine.flnc_bam_pbi,
          max_fuzzy_junction = isocollapse_max_fuzzy_junction,
          min_aln_identity = mapping_min_concordance / 100.0,
          min_aln_coverage = mapping_min_coverage / 100.0,
          nproc = nproc,
          log_level = log_level
      }

      call pbreports.isoseq_mapping as isoseq_mapping_report {
        input:
          collapse_fasta_or_fastq = isoseq3_collapse.fasta,
          collapse_report_json = isoseq3_collapse.report
      }
    }
  }

  output {
    File flnc_bam = isoseq3_refine.flnc_bam
    File flnc_report = isoseq3_refine.report_csv
    File report_barcode_isoseq3 = pbreports_barcode_isoseq3.report
    File? report_isoseq = isoseq_report.report
    File? polish_report_csv = isoseq3_cluster.report_csv
    File? barcode_overview_report = isoseq3_summarize.report_csv
    File? hq_fasta = bam2fasta_transcripts.hq_fasta
    File? lq_fasta = bam2fasta_transcripts.lq_fasta
    File? hq_aln_bam = consolidate_transcriptaln.mapped_bam
    File? hq_aln_bai = consolidate_transcriptaln.mapped_bam_bai
    File? collapse_fasta = isoseq3_collapse.fasta
    File? collapse_gff = isoseq3_collapse.gff
    File? collapse_group = isoseq3_collapse.group
    File? collapse_abundance = isoseq3_collapse.abundance
    File? collapse_readstat = isoseq3_collapse.readstat
    File? report_isoseq_mapping = isoseq_mapping_report.report
  }
}
