# Resequencing using pbmm2 and gcpp

version 1.0

import "wf_mapping.wdl"
import "wf_consensus.wdl"
import "wf_consensus_reports.wdl"
import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/gcpp.wdl" as gcpp

workflow pb_resequencing {
  input {
    File eid_subread
    File eid_ref_dataset
    String consensus_algorithm = "arrow"
    String dataset_filters = ""
    String? mapping_biosample_name
    Float mapping_min_concordance = 70
    Int mapping_min_length = 50
    Int downsample_factor = 0
    Boolean consolidate_aligned_bam = false
    Boolean extract_unmapped_bam = false
    String? mapping_pbmm2_overrides

    Int nproc = 8
    String log_level = "INFO"
    String? tmp_dir
    Int max_nchunks = 100

    # this is so we can call this from SAT workflow
    Boolean internal_enable_consolidation = true
  }

  # some internal settings
  String mapping_stats_module = "mapping_stats"
  String coverage_report_module = "coverage"

  call pbcoretools.dataset_absolutize as update_subreads {
    input:
      dataset = eid_subread,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_absolutize as update_reference {
    input:
      dataset = eid_ref_dataset,
      log_level = log_level,
      nproc = 1
  }

  call pbcoretools.dataset_filter {
    input:
      dataset = update_subreads.updated_xml,
      filters = dataset_filters,
      downsample_factor = downsample_factor,
      nproc = 1,
      log_level = log_level
  }

  call wf_mapping.mapping {
    input:
      reads = dataset_filter.filtered,
      reference = update_reference.updated_xml,
      mapping_stats_module = mapping_stats_module,
      coverage_report_module = coverage_report_module,
      biosample_name = mapping_biosample_name,
      min_concordance = mapping_min_concordance,
      min_length = mapping_min_length,
      pbmm2_overrides = mapping_pbmm2_overrides,
      run_coverage = false,
      run_bam_auto_consolidation = false,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level,
      tmp_dir = tmp_dir
  }

  # the coverage tasks are run in the main workflow because they are blocking
  # otherwise (due to the way Cromwell executes sub-workflows)
  call pbreports.summarize_coverage as summarize_coverage {
    input:
      mapped = mapping.mapped,
      reference = update_reference.updated_xml,
      nproc = nproc
  }

  call pbreports.coverage as coverage_report {
    input:
      reference = update_reference.updated_xml,
      coverage_gff = summarize_coverage.coverage_gff,
      nproc = nproc
  }

  if (internal_enable_consolidation) {
    call pbcoretools.auto_consolidate_alignments {
      input:
        mapped = mapping.mapped,
        force_consolidate = consolidate_aligned_bam,
        tmp_dir = tmp_dir,
        log_level = log_level
    }
  }

  if (extract_unmapped_bam) {
    call pbcoretools.extract_unmapped_reads {
      input:
        mapped = mapping.mapped,
        unmapped = dataset_filter.filtered,
        nproc = nproc,
        log_level = log_level
    }
  }

  call wf_consensus.consensus {
    input:
      alignments = mapping.mapped,
      reference = update_reference.updated_xml,
      consensus_algorithm = consensus_algorithm,
      log_level = log_level,
      nproc = nproc,
      max_nchunks = max_nchunks
  }

  call wf_consensus_reports.consensus_reports {
    input:
      reference = update_reference.updated_xml,
      coverage_gff = summarize_coverage.coverage_gff,
      variants_gff = consensus.variants_gff
  }

  output {
    File mapped = mapping.mapped
    File? report_mapping_stats = mapping.report_mapping_stats
    File coverage_gff = summarize_coverage.coverage_gff
    File report_coverage = coverage_report.report
    File consensus_fasta = consensus.consensus_fasta
    File consensus_fastq = consensus.consensus_fastq
    File variants_gff = consensus.variants_gff
    File variants_vcf = consensus.variants_vcf
    File consensus_gff = consensus_reports.consensus_gff
    File report_variants = consensus_reports.report_variants
    File report_top_variants = consensus_reports.report_top_variants
    File? mapped_bam_datastore = auto_consolidate_alignments.datastore
    File? unmapped_bam = extract_unmapped_reads.unmapped_bam
  }
}
