version 1.0

# Split any BAM-based dataset by holeNumber range (real for Subreads or CCS,
# synthetic for Transcripts)
task split_dataset {
  input {
    File ds_in
    String split_mode = "zmws"

    Int? max_nchunks
    Int? target_size

    Int nproc = 2
    String log_level = "DEBUG"
  }

  command {
    touch .NOARCHIVE
    dataset \
      --log-level ${log_level} \
      split \
      --${split_mode} \
      --simple-chunk-ids \
      ${"--maxChunks " + max_nchunks} \
      ${"--targetSize " + target_size} \
      --outdir . \
      ${ds_in}
  }
  runtime {
    # single-processor but may have a large memory footprint
    cpu: 2
  }
  output {
    # Note that unlike pbsmrtpipe tooling, these currently retain the basename
    # prefix of the input dataset.  We can override this with the new --prefix
    # option if desired.
    Array[File] chunks = glob("*set.xml")
    Int nchunks = length(chunks)
  }
}

task split_alignments {
  input {
    File mapped
    Int? max_nchunks
    Int? target_size

    Int nproc = 2
    String log_level = "DEBUG"
  }
  command {
    touch .NOARCHIVE
    dataset \
      --log-level ${log_level} \
      split \
      ${"--maxChunks " + max_nchunks} \
      ${"--targetSize " + target_size} \
      --contigs --breakContigs \
      --outdir . \
      ${mapped}
  }
  runtime {
    # single-processor but may have a large memory footprint
    cpu: 2
  }
  output {
    Array[File] chunks = glob("*set.xml")
    Int nchunks = length(chunks)
  }
}

# Gather chunks of any dataset type.  Unlike split, this requires that we
# explicitly set the dataset type and extension.
task gather_datasets {
  input {
    Array[File] chunks
    String dataset_ext
    String? dataset_name
    String prefix = "gathered"
    Boolean no_sub_datasets = false

    Int nproc = 2
    String log_level = "DEBUG"
  }
  command {
    dataset \
      --log-level ${log_level} \
      --strict \
      create \
      ${"--name " + dataset_name} \
      ${true="--no-sub-datasets" false="" no_sub_datasets} \
      --unique-collections \
      ${prefix}${dataset_ext} \
      ${sep=" " chunks}
  }
  runtime {
    # this runs serially but may have a large memory footprint
    cpu: 2
  }
  output {
    File gathered = "${prefix}${dataset_ext}"
  }
}

task gather_generic {
  input {
    Array[File] chunks
    String output_file_name
    String? gather_args

    Int nproc = 1
    String log_level = "DEBUG"
  }
  command {
    python \
      -m pbcoretools.tasks2.gather \
      --log-level ${log_level} \
      ${gather_args} \
      ${output_file_name} \
      ${sep=" " chunks}
  }
  runtime {
    cpu: 1
  }
  output {
    File gathered = "${output_file_name}"
  }
}

task chunk_reads_for_lima {
  input {
    File reads
    Boolean lima_peek_guess
    Int? target_size
    Int? max_nchunks

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python -m \
      pbcoretools.tasks2.chunk_reads_for_lima \
      --log-level ${log_level} \
      ${true="--lima-peek-guess" false="" lima_peek_guess} \
      ${"--targetSize " + target_size} \
      ${"--maxChunks " + max_nchunks} \
      ${reads}
  }
  runtime {
    cpu: 1
  }
  output {
    Array[File] chunks = glob("*set.xml")
    Int nchunks = length(chunks)
  }
}

task gather_lima_datasets {
  input {
    Array[File] datastores

    Int nproc
    String log_level = "INFO"
  }
  command {
    python -m \
      pbcoretools.tasks2.gather_lima_datasets \
      --log-level ${log_level} \
      --nproc ${nproc} \
      lima.datastore.json \
      ${sep=" " datastores} \
  }
  runtime {
    cpu: nproc
  }
  output {
    File gathered = "lima.datastore.json"
  }
}

task gather_bam {
  input {
    Array[File] bam_files
    String output_file_name = "merged.bam"

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    pbmerge -o ${output_file_name} ${sep=" " bam_files}
  }
  runtime {
    cpu: 1
  }
  output {
    File merged = "${output_file_name}"
  }
}

task gather_fofn {
  input {
    Array[File] fofn_files

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python -m \
      pbcoretools.tasks2.gather_fofn \
      ${sep=" " fofn_files}
  }
  runtime {
    cpu: 1
  }
  output {
    File merged = "merged.bam"
  }
}
