version 1.0

task refine {
  input {
    File fl_ccs
    File barcodes

    Boolean require_polya = true

    Int nproc
    String log_level = "INFO"
  }

  command {
    isoseq3 \
      refine \
      --log-level ${log_level} \
      -j ${nproc} \
      ${true="--require-polya" false="" require_polya} \
      ${fl_ccs} \
      ${barcodes} \
      flnc.consensusreadset.xml \
      flnc.filter_summary.json \
      flnc.report.csv
  }
  runtime {
    cpu: nproc
  }
  output {
    File flnc_ccs = "flnc.consensusreadset.xml"
    File flnc_bam = "flnc.bam"
    File flnc_bam_pbi = "flnc.bam.pbi"
    File filter_summary = "flnc.filter_summary.json"
    File report_csv = "flnc.report.csv"
  }
}

task cluster {
  input {
    File flnc_ccs

    Int nproc
    String log_level = "INFO"
  }

  command {
    isoseq3 \
      cluster \
      --log-level ${log_level} \
      ${flnc_ccs} \
      unpolished.transcriptset.xml \
      -j ${nproc}
  }
  runtime {
    cpu: nproc
  }
  output {
    File unpolished = "unpolished.transcriptset.xml"
  }
}

task cluster_ccsonly {
  input {
    File flnc_ccs

    Int min_subreads_split
    Int nproc
    String log_level = "INFO"
  }

  command {
    isoseq3 \
      cluster \
      --log-level ${log_level} \
      ${flnc_ccs} \
      --use-qvs --min-subreads-split ${min_subreads_split} \
      unpolished.transcriptset.xml \
      -j ${nproc} \
    && \
    dataset create --type TranscriptSet \
       --generateIndices \
       unpolished.hq.transcriptset.xml \
       unpolished.hq.bam \
    && \
    dataset create --type TranscriptSet \
       --generateIndices \
       unpolished.lq.transcriptset.xml \
       unpolished.lq.bam
  }
  runtime {
    cpu: nproc
  }
  output {
    File unpolished = "unpolished.transcriptset.xml"
    File report_csv = "unpolished.cluster_report.csv"
    File hq_bam = "unpolished.hq.bam"
    File lq_bam = "unpolished.lq.bam"
    File hq_transcripts = "unpolished.hq.transcriptset.xml"
    File lq_transcripts = "unpolished.lq.transcriptset.xml"
  }
}

task polish {
  input {
    File unpolished
    File subreads

    Int nproc
    String log_level = "INFO"
  }

  command {
    isoseq3 \
      polish \
      --log-level ${log_level} \
      ${unpolished} \
      ${subreads} \
      polished.transcriptset.xml \
      polished.cluster_report.csv \
      -j ${nproc}
  }
  runtime {
    cpu: nproc
  }
  output {
    File polished = "polished.transcriptset.xml"
    File report_csv = "polished.cluster_report.csv"
  }
}

task summarize {
  input {
    File transcriptset
    File barcodeset

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    isoseq3 summarize \
      --log-level ${log_level} \
      ${transcriptset} \
      ${barcodeset} \
      summary.csv
  }
  runtime {
    cpu: 1
  }
  output {
    File report_csv = "summary.csv"
  }
}

task extract_hq_transcripts {
  input {
    File polished
    Float hq_cutoff = 0.99

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    dataset --log-level ${log_level} filter ${polished} hq.transcriptset.xml 'rq >= ${hq_cutoff}' && \
    dataset --log-level ${log_level} filter ${polished} lq.transcriptset.xml 'rq < ${hq_cutoff}'
  }
  runtime {
    cpu: 1
  }
  output {
    File hq_transcripts = "hq.transcriptset.xml"
    File lq_transcripts = "lq.transcriptset.xml"
  }
}

task bam2fasta_transcripts {
  input {
    File hq_transcripts
    File lq_transcripts
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbcoretools.tasks.bam2fasta_transcripts \
      --log-level ${log_level} \
      ${hq_transcripts} \
      ${lq_transcripts} \
      ${subreads} \
      hq_transcripts.fasta \
      lq_transcripts.fasta
  }
  runtime {
    cpu: 1
  }
  output {
    File hq_fasta = "hq_transcripts.fasta"
    File lq_fasta = "lq_transcripts.fasta"
  }
}

task bam2fastq_transcripts {
  input {
    File hq_transcripts
    File lq_transcripts
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbcoretools.tasks.bam2fastq_transcripts \
      --log-level ${log_level} \
      ${hq_transcripts} \
      ${lq_transcripts} \
      ${subreads} \
      hq_transcripts.fastq \
      lq_transcripts.fastq
  }
  runtime {
    cpu: 1
  }
  output {
    File hq_fastq = "hq_transcripts.fastq"
    File lq_fastq = "lq_transcripts.fastq"
  }
}

task collapse {
  input {
    File hq_transcriptaln_bam
    File flnc_bam
    File flnc_bam_pbi

    Float min_aln_coverage = 0.99
    Float min_aln_identity = 0.95
    Int max_fuzzy_junction = 5

    Int nproc
    String log_level = "INFO"
  }

  command {
    ls ${flnc_bam_pbi} \
      && \
    isoseq3 collapse \
      --log-level ${log_level} \
      -j ${nproc} \
      --min-aln-coverage ${min_aln_coverage} \
      --min-aln-identity ${min_aln_identity} \
      --max-fuzzy-junction ${max_fuzzy_junction} \
      ${hq_transcriptaln_bam} \
      ${flnc_bam} \
      collapse_isoforms.gff \
      && \
    mv collapse_isoforms.report.json collapse_isoforms_report.json # To prevent SL autoload .report.json
  }
  runtime {
    cpu: nproc
  }
  output {
    File fastq = "collapse_isoforms.fastq"
    File gff = "collapse_isoforms.gff"
    File abundance = "collapse_isoforms.abundance.txt"
    File group = "collapse_isoforms.group.txt"
    File readstat = "collapse_isoforms.read_stat.txt"
    File report = "collapse_isoforms_report.json"
  }
}

task collapse_ccsonly {
  input {
    File hq_transcriptaln_bam
    File flnc_bam
    File flnc_bam_pbi

    Float min_aln_coverage = 0.99
    Float min_aln_identity = 0.95
    Int max_fuzzy_junction = 5

    Int nproc
    String log_level = "INFO"
  }

  command {
    ls ${flnc_bam_pbi} \
      && \
    isoseq3 collapse \
      --log-level ${log_level} \
      -j ${nproc} \
      --min-aln-coverage ${min_aln_coverage} \
      --min-aln-identity ${min_aln_identity} \
      --max-fuzzy-junction ${max_fuzzy_junction} \
      ${hq_transcriptaln_bam} \
      ${flnc_bam} \
      collapse_isoforms.gff
  }
  runtime {
    cpu: nproc
  }
  output {
    File fasta = "collapse_isoforms.fasta"  # CCSONLY transcripts do not contain QVs
    File gff = "collapse_isoforms.gff"
    File abundance = "collapse_isoforms.abundance.txt"
    File group = "collapse_isoforms.group.txt"
    File readstat = "collapse_isoforms.read_stat.txt"
    File report = "collapse_isoforms.report.json"
  }
}

task check_reads_for_lima {
  input {
    File ccs
    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbcoretools.tasks2.check_reads_for_lima \
      ${ccs}
  }
  runtime {
    cpu: 1
  }
  output {
  }
}
 
