version 1.0

# This task relies on Cromwell being configured for soft-link behavior, since
# the dataset command will automatically resolve the real path on disk.
task dataset_absolutize {
  input {
    # This is only optional because WOMtool is fundamentally broken, or maybe
    # I just don't understand WDL:
    # https://gatkforums.broadinstitute.org/wdl/discussion/23250/wdl-1-0-wont-let-me-call-an-optional-task-with-an-optional-input
    File? dataset

    Int nproc = 1
    String log_level = "INFO"
  }
  # workaround for File? and basename
  Array[File?] datasets_ = [dataset]
  File dataset_ = select_first(datasets_)
  command {
    touch .NOARCHIVE
    set -vex
    dataset \
      --skipCounts \
      --log-level ${log_level} \
      absolutize \
      ${dataset_} \
      --outdir .
  }
  runtime {
    backend: "Local"
  }
  output {
    File updated_xml = basename(dataset_)
  }
}

task dataset_filter {
  input {
    File dataset
    String dataset_ext = ".subreadset.xml"
    String filters
    Int? downsample_factor
    Int? min_read_length

    Int nproc = 2
    String log_level = "INFO"
  }

  command {
    touch .NOARCHIVE
    set -vex
    python \
      -m pbcoretools.tasks2.dataset_filter \
      --log-level ${log_level} \
      ${dataset} \
      filtered${dataset_ext} \
      '${filters}' \
      ${"--min-read-length " + min_read_length} \
      ${"--downsample " + downsample_factor}
  }
  runtime {
    # this may have significant memory overhead for large datasets
    cpu: 2
  }
  output {
    File filtered = "filtered${dataset_ext}"
  }
}

task auto_ccs_outputs {
  input {
    File ccsxml
    String mode

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbcoretools.tasks2.auto_ccs_outputs \
      --log-level ${log_level} \
      ${mode} \
      ${ccsxml} \
      ${mode}.datastore.json
  }
  runtime {
    cpu: 1
  }
  output {
    File datastore = "${mode}.datastore.json"
  }
}

task auto_ccs_outputs_barcoded {
  input {
    File datastore_in

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python \
      -m pbcoretools.tasks.auto_ccs_outputs_barcoded \
      --log-level ${log_level} \
      ${datastore_in} \
      ccs_demultiplexed_outputs.datastore.json
  }
  runtime {
    cpu: 1
  }
  output {
    File datastore = "ccs_demultiplexed_outputs.datastore.json"
  }
}

task consolidate_alignments {
  input {
    File mapped
    String dataset_ext = ".alignmentset.xml"
    String? tmp_dir

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    ${"TMPDIR=" + tmp_dir} \
    dataset \
      --log-level ${log_level} \
      consolidate \
      ${mapped} \
      mapped.bam \
      mapped${dataset_ext}
  }
  runtime {
    cpu: 1
  }
  output {
    File consolidated = "mapped${dataset_ext}"
    File mapped_bam = "mapped.bam"
    File mapped_bam_bai = "mapped.bam.bai"
  }
}

# More flexible consolidation task that will run only if combined BAM files
# are less than 10GB
task auto_consolidate_alignments {
  input {
    File mapped
    Boolean force_consolidate = false
    String? tmp_dir

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    # We specify TMPDIR despite --noTmp b/c the XML will be written to TMPDIR anyway.
    # --noTmp refers only to BAM.
    set -vex

    ${"TMPDIR=" + tmp_dir} \
    python -m pbcoretools.tasks2.auto_consolidate \
      --log-level ${log_level} \
      --noTmp \
      ${true="--force" false="" force_consolidate} \
      ${mapped} \
      --datastore mapped.datastore.json \
      mapped.bam
  }
  runtime {
    cpu: 1
  }
  output {
    # this avoids call-caching bulky files in Cromwell
    File? datastore = "mapped.datastore.json"
  }
}


task consolidate_transcripts {
  input {
    File subreads
    File hq_transcripts
    File lq_transcripts

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python -m pbcoretools.tasks.consolidate_transcripts \
      --log-level ${log_level} \
      ${subreads} \
      ${hq_transcripts} \
      ${lq_transcripts} \
      combined.hq.transcriptset.xml \
      combined.lq.transcriptset.xml \
      resources.hq.json \
      resources.lq.json \
      && \
    dataset \
      --log-level ${log_level} \
      absolutize \
      combined.hq.transcriptset.xml \
      && \
    dataset \
      --log-level ${log_level} \
      absolutize \
      combined.lq.transcriptset.xml \
  }
  runtime {
    cpu: 1
  }
  output {
    File out_hq_transcripts = "combined.hq.transcriptset.xml"
    File out_lq_transcripts = "combined.lq.transcriptset.xml"
  }
}

task update_consensus_reads {
  input {
    File ccs_in
    File subreads_in
    Boolean use_run_design_uuid

    Int nproc = 1
    String log_level = "INFO"
  }

  command {
    python -m pbcoretools.tasks2.update_consensus_reads \
      --log-level ${log_level} \
      ${true="--use-run-design-uuid" false="" use_run_design_uuid} \
      ${ccs_in} \
      ${subreads_in} \
      final.consensusreadset.xml
  }
  runtime {
    cpu: 1
  }
  output {
    File ccsxml = "final.consensusreadset.xml"
  }
}

task extract_unmapped_reads {
  input {
    File mapped
    File unmapped
    String bam_ext = ".subreads.bam"

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    bamsieve \
      --log-level ${log_level} \
      --blacklist ${mapped} \
      --subreads \
      ${unmapped} \
      unaligned${bam_ext}
  }
  runtime {
    cpu: 1
  }
  output {
    File unmapped_bam = "unaligned${bam_ext}"
  }
}

task pbvalidate {
  input {
    File data
    Boolean quick_mode = false

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    pbvalidate \
      --log-level ${log_level} \
      --alarms alarms.json \
      ${true="--quick" false="" quick_mode} \
      ${data}
  }
  runtime {
    cpu: 1
  }
  # pbvalidate doesn't actually write anything, this just gives us a way to
  # create a graph dependency
  output {
    File data_out = data
  }
}

task split_laa_fastq {
  input {
    File consensus
    File chimeras
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbcoretools.tasks2.split_laa_fastq \
      --log-level ${log_level} \
      ${consensus} \
      ${chimeras} \
      ${subreads} \
      consensus_fastq.zip \
      chimeras_fastq.zip
  }
  runtime {
    cpu: 1
  }
  output {
    File consensus_fastq_zip = "consensus_fastq.zip"
    File chimeras_fastq_zip = "chimeras_fastq.zip"
  }
}

task make_combined_laa_zip {
  input {
    File consensus
    File summary_csv
    File subreads

    Int nproc = 1
    String log_level = "INFO"
  }
  command {
    python \
      -m pbcoretools.tasks2.make_combined_laa_zip \
      ${consensus} \
      ${summary_csv} \
      ${subreads} \
      combined_results.zip
  }
  runtime {
    cpu: 1
  }
  output {
    File combined_zip = "combined_results.zip"
  }
}
