version 1.0

import "tasks/pbreports.wdl" as pbreports
import "tasks/chunking.wdl" as chunking

task graph_mapping {
    input {
        File reference
        File gfa
        File reads
        String out_fn
        String param_overrides
        Int nproc
        Float min_idt
        Int min_length
    }
    command <<<
        graph_param="-g ~{gfa}"
        if [ -z "~{gfa}" ]; then
            graph_param=""
        fi

        # Downstream Pbcore tools cannot work with unmapped alignments, so we need to filter them out here.
        raptor -t ~{nproc} --out-fmt bam -r ~{reference} ${graph_param} -q ~{reads} --min-idt ~{min_idt} --min-qlen ~{min_length} ~{param_overrides} -o - | samtools view -bh -F 0x4 > ~{out_fn}.bam
        samtools sort --threads ~{nproc} -o ~{out_fn}.sorted.bam ~{out_fn}.bam
        samtools index ~{out_fn}.sorted.bam
        rm -f ~{out_fn}.bam
        pbindex ~{out_fn}.sorted.bam
        dataset create ~{out_fn} ~{out_fn}.sorted.bam
    >>>
    runtime {
        cpu: nproc
    }
    output {
        File mapped = "${out_fn}"
    }
}

workflow RaptorGraphMapping {
    input {
        File reads_xml
        # Raptor can't read referencesets at the moment, just plain sequence files.
        File reference
        # The referenceset is needed for reports.
        File referenceset_xml
        File gfa
        String param_overrides
        String alignment_ext = ".alignmentset.xml"
        String mapping_stats_module = "mapping_stats"
        String coverage_report_module = "coverage"
        Float min_concordance = 70
        Int min_length = 50

        Int nproc = 16
        Int? max_nchunks
        Int? target_size

        String log_level = "INFO"
    }

    call chunking.split_dataset as split_reads {
        input:
            ds_in = reads_xml,
            max_nchunks = max_nchunks,
            target_size = target_size
    }

    scatter (chunk in split_reads.chunks) {
        call graph_mapping {
            input:
                reference = reference,
                gfa = gfa,
                reads = chunk,
                out_fn = "mapped.alignmentset.xml",
                param_overrides = param_overrides,
                nproc = nproc,
                min_idt = min_concordance,
                min_length = min_length,
        }
    }

    call chunking.gather_datasets as gather_alignments {
        input:
            chunks = graph_mapping.mapped,
            dataset_name = "'Mapped Reads'",
            dataset_ext = alignment_ext
    }

    call pbreports.mapping_stats as mapping_stats {
        input:
            mapped = gather_alignments.gathered,
            unmapped = reads_xml,
            report_module = mapping_stats_module,
            nproc = nproc,
            log_level = log_level
    }

    call pbreports.summarize_coverage as summarize_coverage {
        input:
            mapped = gather_alignments.gathered,
            reference = referenceset_xml,
            nproc = nproc,
            log_level = log_level
    }

    call pbreports.coverage as coverage_report {
        input:
            reference = referenceset_xml,
            coverage_gff = summarize_coverage.coverage_gff,
            report_module = coverage_report_module,
            nproc = nproc,
            log_level = log_level
    }

    output {
        File mapped = gather_alignments.gathered
        File report_mapping_stats = mapping_stats.report
        File coverage_gff = summarize_coverage.coverage_gff
        File report_coverage = coverage_report.report
    }
}
