# simple mapping pipeline, also used in resequencing

version 1.0

import "tasks/chunking.wdl" as chunking
import "tasks/pbreports.wdl" as pbreports
import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/pbmm2.wdl" as mapping_tasks

workflow mapping {
  input {
    File reads
    File reference
    String alignment_ext = ".alignmentset.xml"
    String mapping_stats_module = "mapping_stats"
    String coverage_report_module = "coverage"
    String? biosample_name
    String? preset_mode
    Float min_concordance = 70
    Int min_length = 50
    Boolean hq_mode = false
    Boolean zmw_mode = false
    Boolean? median_filter = false
    Boolean? strip = false
    Boolean? split_by_sample = false
    String? pbmm2_overrides
    Boolean run_coverage = true
    Boolean run_mapping_stats = true
    Boolean run_bam_auto_consolidation = false # this just enables auto behavior
    Boolean consolidate_aligned_bam = false # this is the user-facing option

    Int nproc = 8
    Int? max_nchunks
    Int? target_size
    String log_level = "INFO"
    String? tmp_dir
  }

  call chunking.split_dataset as split_reads {
    input:
      ds_in = reads,
      max_nchunks = max_nchunks,
      target_size = target_size
  }

  scatter (chunk in split_reads.chunks) {
    call mapping_tasks.pbmm2_align as pbmm2_align {
      input:
        unmapped = chunk,
        reference = reference,
        aln_ext = alignment_ext,
        biosample_name = biosample_name,
        preset_mode = preset_mode,
        min_concordance = min_concordance,
        min_length = min_length,
        hq_mode = hq_mode,
        zmw_mode = zmw_mode,
        median_filter = median_filter,
        strip = strip,
        split_by_sample = split_by_sample,
        pbmm2_overrides = pbmm2_overrides,
        nproc = nproc,
        log_level = log_level,
        tmp_dir = tmp_dir
    }
  }

  call chunking.gather_datasets as gather_alignments {
    input:
      chunks = pbmm2_align.mapped,
      dataset_name = "'Mapped Reads'",
      dataset_ext = alignment_ext,
      prefix = "mapped",
      no_sub_datasets = true
  }

  # for workflows like pb_assembly_microbial that run two separate mapping
  # steps, we only want to generate one report
  if (run_mapping_stats) {
    call pbreports.mapping_stats as mapping_stats {
      input:
        mapped = gather_alignments.gathered,
        unmapped = reads,
        report_module = mapping_stats_module,
        nproc = nproc,
        log_level = log_level
    }
  }

  # warning: this is slow, so some workflows run these tasks separately
  if (run_coverage) {
    call pbreports.summarize_coverage as summarize_coverage {
      input:
        mapped = gather_alignments.gathered,
        reference = reference,
        nproc = nproc,
        log_level = log_level
    }

    call pbreports.coverage as coverage_report {
      input:
        reference = reference,
        coverage_gff = summarize_coverage.coverage_gff,
        report_module = coverage_report_module,
        nproc = nproc,
        log_level = log_level
    }
  }

  # this is even slower, so it is also usually called elsewhere
  if (run_bam_auto_consolidation) {
    call pbcoretools.auto_consolidate_alignments {
      input:
        mapped = gather_alignments.gathered,
        force_consolidate = consolidate_aligned_bam,
        log_level = log_level,
        tmp_dir = tmp_dir
    }
  }

  output {
    File mapped = gather_alignments.gathered
    File? report_mapping_stats = mapping_stats.report
    File? coverage_gff = summarize_coverage.coverage_gff
    File? report_coverage = coverage_report.report
    File? mapped_bam_datastore = auto_consolidate_alignments.datastore
  }
}
