# Common workflow for pb_sv_ccs and pb_sv_clr

version 1.0

import "tasks/pbcoretools.wdl" as pbcoretools
import "tasks/chunking.wdl" as chunking
import "wf_mapping.wdl"
import "tasks/sv.wdl" as sv
import "tasks/pbreports.wdl" as pbreports

# Support scatter + gather for mapping, tandem repeat finder, pbsv discover, bam merge

workflow pbsv_intl {
  # Note all datasets (ccs, subreads, reference) must be absolutized in
  # pbsv_clr.wdl or pbsv_ccs.wdl
  input {
    File reads
    File reference

    # Mapping parameters
    String mapping_alignment_ext
    Int mapping_min_length = 50
    Float mapping_min_concordance = 70
    Boolean mapping_median_filter = true
    Boolean mapping_strip = true
    Boolean mapping_split_by_sample = true
    String? mapping_biosample_name
    String? mapping_preset_mode
    String? mapping_pbmm2_overrides

    # pbsv call parameters
    String chunk_length = "1M"
    Int min_sv_length = 20
    Int min_cnv_length = 1000
    Int min_percent_reads = 20
    Int min_reads_one_sample = 2
    Int min_reads_all_samples = 2
    Int min_reads_per_strand_all_samples = 1

    # Workflow resource configuration parameters
    Int nproc = 1
    String log_level = "INFO"
    Int max_nchunks = 100
  }

  call sv.pbsv_split_ref {
    input: 
      reference = reference,
      log_level = log_level,
      nproc = 1
  }

  call sv.pbsv_scatter_tandem_repeat_finder {
    input:
      split_ref_csv = pbsv_split_ref.split_ref_csv,
      max_nchunks = max_nchunks,
      log_level = log_level,
      nproc = 1
  }

  scatter (chunk in pbsv_scatter_tandem_repeat_finder.chunks) {
    call sv.pbsv_tandem_repeat_finder {
      input: 
        reference = reference,
        split_ref_csv = chunk,
        log_level = log_level,
        nproc = nproc
    }
  }
  
  call sv.pbsv_gather_tandem_repeat_finder {
    input:
      chunks = pbsv_tandem_repeat_finder.tandem_repeat_bed,
      log_level = log_level
  }

  call wf_mapping.mapping {
    input:
      reads = reads,
      reference = reference,
      alignment_ext = mapping_alignment_ext,
      biosample_name = mapping_biosample_name,
      min_concordance = mapping_min_concordance,
      median_filter = mapping_median_filter,
      strip = mapping_strip,
      split_by_sample = mapping_split_by_sample,
      preset_mode = mapping_preset_mode,
      min_length = mapping_min_length,
      pbmm2_overrides = mapping_pbmm2_overrides,
      nproc = nproc,
      max_nchunks = max_nchunks,
      log_level = log_level
  }


  call sv.dataset_to_datastore {  # alignmentset.xml -> alignments.datastore.json
    input:
      dataset = mapping.mapped,
      log_level = log_level
  }

  call sv.pbsv_scatter_align_datastore {
    input:
      align_datastore_json = dataset_to_datastore.datastore,
      max_nchunks = max_nchunks,
      log_level = log_level
  }

  scatter (chunk in pbsv_scatter_align_datastore.chunks) {
    call sv.pbsv_align_json_to_svsig {  # alignments -> svsig
      input: 
        align_datastore_json = chunk,
        tandem_repeat_bed = pbsv_gather_tandem_repeat_finder.gathered,
        log_level = log_level,
        nproc = nproc
    }
  }

  call chunking.gather_generic as pbsv_gather_svsig {
    input:
      chunks = pbsv_align_json_to_svsig.svsig_gz_fofn,
      output_file_name = "gathered.pbsv.svsig.fofn",
      log_level = log_level
  }

  call sv.pbsv_call {  # svsig -> structural_variants.vcf, and json reports
    input:
      reference = reference,
      svsig_gz_fofn = pbsv_gather_svsig.gathered,
      chunk_length = chunk_length,
      min_sv_length = min_sv_length,
      min_cnv_length = min_cnv_length,
      min_percent_reads = min_percent_reads,
      min_reads_one_sample = min_reads_one_sample,
      min_reads_all_samples = min_reads_all_samples,
      min_reads_per_strand_all_samples = min_reads_per_strand_all_samples,
      nproc = nproc,
      log_level = log_level
  }

  call sv.pbsv_scatter_align_datastore_by_sample as scatter_datastore {
    # one chunk for each biosample
    input: 
      align_datastore_json = dataset_to_datastore.datastore,
      max_nchunks = max_nchunks,
      log_level = log_level
  }

  scatter (i_chunk in range(length(scatter_datastore.chunks))) {
    call sv.pbsv_merge_alignments_by_sample {
      # output one alignments.bam for each sample
      input:
        align_datastore_json = scatter_datastore.chunks[i_chunk],
        i_chunk = i_chunk,
        nproc = nproc,
        log_level = log_level
    }
  }

  call chunking.gather_generic as gather_alignments {
    input:
      chunks = pbsv_merge_alignments_by_sample.alignments_by_sample_datastore,
      output_file_name = "alignments_by_sample.datastore.json",
      log_level = log_level
  }

  call pbreports.structural_variants_2_report {
    input:
      lengths_json = pbsv_call.lengths_report,
      summary_json = pbsv_call.summary_report,
      nproc = 1,
      log_level = log_level
  }

  output {
    File variants = pbsv_call.variants
    File alignments_by_sample_datastore = gather_alignments.gathered
    File report = structural_variants_2_report.report
  }
}
